# this file contains code for our AST engine that parses given code, indexes it and then can be queried
# to get things. It's nothing as fancy as SCP, but it will need to have better UX than on_functions.
# Here's a list of usecases till now:
# - find a given object (str) in the given file and return all the relevant information about it

from functools import lru_cache
import os
import re
import ast as A
from enum import Enum
from typing import List, Set, Union

class IndexTypes(Enum):
  FUNCTION = A.FunctionDef
  CLASS = A.ClassDef
  VARIABLE = A.Name
  IMPORT = A.Import
  IMPORT_FROM = A.ImportFrom
  ASSIGN = A.Assign
  EXPRESSION = A.Expr
  IF = A.If
  FILE = "file"

  def all():
    return [
      IndexTypes.FUNCTION, IndexTypes.CLASS, IndexTypes.VARIABLE, IndexTypes.IMPORT, IndexTypes.IMPORT_FROM,
      IndexTypes.ASSIGN, IndexTypes.EXPRESSION, IndexTypes.IF, IndexTypes.FILE
    ]

class Astea:
  def __init__(
    self,
    fname: str = "",
    *,
    name: str = "",
    type: IndexTypes = None,
    node: A.AST = None,
    code_lines: List[str] = None,
    order_index: int = -1,
  ) -> None:
    """This is an AST node, that can be called upon itself to traverse in a natural human way. Only `fname` is an important
    argument, others will be populated automatically.
    Args:
      fname (str): the file name to parse
    """

    if fname:
      assert os.path.exists(fname), f"file/folder {fname} does not exist"
      if os.path.isdir(fname):
        self._fname = os.path.join(fname, "__init__.py")
      type = IndexTypes.FILE
      with open(fname, 'r') as f:
        code = f.read()
        node: A.Module = A.parse(code)
    elif name:
      pass
    else:
      raise ValueError("Must provide either fname or name")
    
    self.name = name
    self.type = type
    self.node = node
    self.code_lines = code_lines
    self.order_index = order_index

  def __repr__(self) -> str:
    return f"{self.type.name} {self.name}"

  def __hash__(self) -> int:
    return repr(self).__hash__()

  @property
  @lru_cache(1)
  def index(self) -> List['Astea']:
    items = set()
    if hasattr(self.node, 'body'):
      for i, n in enumerate(self.node.body):
        if type(n) == A.FunctionDef:
          tea = Astea(name=n.name, type=IndexTypes.FUNCTION, node=n, code_lines=self.code_lines, order_index=i)
          items.add(tea)
        elif type(n) == A.ClassDef:
          tea = Astea(name=n.name, type=IndexTypes.CLASS, node=n, code_lines=self.code_lines, order_index=i)
          items.add(tea)
        elif type(n) == A.Import:
          for j, nn in enumerate(n.names):
            tea = Astea(name=nn.name, type=IndexTypes.IMPORT, node=n, code_lines=self.code_lines, order_index=i+j)
            items.add(tea)
        elif type(n) == A.ImportFrom:
          for j, nn in enumerate(n.names):
            tea = Astea(name=nn.name, type=IndexTypes.IMPORT_FROM, node=n, code_lines=self.code_lines, order_index=i+j)
            items.add(tea)
        elif type(n) == A.Expr:
          tea = Astea(name=n.value, type=IndexTypes.EXPRESSION, node=n, code_lines=self.code_lines, order_index=i)
          items.add(tea)
        elif type(n) == A.Assign:
          for j, nn in enumerate(n.targets):
            if hasattr(n, "id"):
              tea = Astea(name=nn.id, type=IndexTypes.VARIABLE, node=n, code_lines=self.code_lines, order_index=i+j)
            else:
              pass
            items.add(tea)
        else:
          continue
    items = sorted(list(items), key=lambda x: x.order_index)
    return items

  def find(self, x: str, types: Union[IndexTypes, List[IndexTypes]] = None) -> list:
    """Find all the instances of x in the current node, and return a list of IndexItems.
    
    - if there is a '.' (dot) in `x` find will try to perform a recursive search, you can escape the dot with a backslash
    """
    if types is None:
      types = IndexTypes.all()
    elif isinstance(types, IndexTypes):
      types = [types]

    # get the sections of the search string
    sections = []
    for s in re.split(r'(?<!\\)\.', x):
      sections.append(s.replace(r'\.', '.'))

    res = []
    node = self
    for i, s in enumerate(sections):
      if i == len(sections) - 1:
        # last section, we need to find the actual item
        for item in node.index:
          if item.name == s and item.type in types:
            res.append(item)
      else:
        # we need to find the next node
        for item in node.index:
          if item.name == s and item.type in types:
            node = item
            break
        else:
          # we didn't find the node, so we can't continue
          break
    return res

  def filter(self, types: Union[IndexTypes, List[IndexTypes]] = None, r: str = "") -> list:
    """Filter the index by the given types"""
    if types is None:
      types = IndexTypes.all()
    elif isinstance(types, IndexTypes):
      types = [types]
    items = [item for item in self.index if item.type in types]
    if r:
      items = [item for item in items if re.search(r, item.name)]
    return items



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: proto/lmao.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10proto/lmao.proto\x12\x04lmao\x1a\x1fgoogle/protobuf/timestamp.proto\"i\n\tLogBuffer\x12\x13\n\x0b\x61gent_token\x18\x01 \x01(\t\x12\x1e\n\x08run_logs\x18\x02 \x03(\x0b\x32\x0c.lmao.RunLog\x12\'\n\tlive_logs\x18\x03 \x03(\x0b\x32\x14.lmao.ServingHTTPLog\"\x86\x01\n\x0eServingHTTPLog\x12\x0c\n\x04path\x18\x01 \x01(\t\x12\x0e\n\x06method\x18\x02 \x01(\t\x12\x13\n\x0bstatus_code\x18\x03 \x01(\x05\x12\x12\n\nlatency_ms\x18\x04 \x01(\x02\x12-\n\ttimestamp\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"[\n\x16ListDeploymentsRequest\x12\x14\n\x0cworkspace_id\x18\x01 \x01(\t\x12\x1a\n\x12serving_id_or_name\x18\x02 \x01(\t\x12\x0f\n\x07page_no\x18\x03 \x01(\x05\"\xb1\x01\n\x17ListDeploymentsResponse\x12\x36\n\x07serving\x18\x01 \x03(\x0b\x32%.lmao.ListDeploymentsResponse.Serving\x12\x13\n\x0btotal_pages\x18\x02 \x01(\x05\x1aI\n\x07Serving\x12\x12\n\nserving_id\x18\x01 \x01(\t\x12\x14\n\x0cserving_name\x18\x02 \x01(\t\x12\x14\n\x0ctotal_models\x18\x03 \x01(\x05\"q\n\x13ListServingsRequest\x12\x14\n\x0cworkspace_id\x18\x01 \x01(\t\x12\x15\n\rdeployment_id\x18\x02 \x01(\t\x12\x0e\n\x06run_id\x18\x03 \x01(\t\x12\x0f\n\x07page_no\x18\x04 \x01(\x05\x12\x0c\n\x04\x64\x65sc\x18\x05 \x01(\x08\"`\n\x14ListServingsResponse\x12\x11\n\tmodel_ids\x18\x01 \x03(\t\x12\x0c\n\x04keys\x18\x02 \x03(\t\x12\x12\n\ncreated_at\x18\x03 \x03(\x03\x12\x13\n\x0btotal_pages\x18\x04 \x01(\x05\"\xd2\x01\n\x07Serving\x12!\n\x05\x61gent\x18\x01 \x01(\x0b\x32\x12.lmao.AgentDetails\x12\x13\n\x0b\x61gent_token\x18\x02 \x01(\t\x12\x12\n\ncreated_at\x18\x03 \x01(\x03\x12\x0e\n\x06\x63onfig\x18\x08 \x01(\t\x12$\n\x06status\x18\t \x01(\x0e\x32\x14.lmao.Serving.Status\x12\x12\n\nupdated_at\x18\n \x01(\x03\"1\n\x06Status\x12\x0b\n\x07NOT_SET\x10\x00\x12\x0b\n\x07RUNNING\x10\x01\x12\r\n\tCOMPLETED\x10\x02\"\xbd\x01\n\x06Record\x12\x0b\n\x03key\x18\x01 \x01(\t\x12)\n\nvalue_type\x18\x02 \x01(\x0e\x32\x15.lmao.Record.DataType\x12\x0c\n\x04step\x18\x03 \x01(\x03\x12\x12\n\nfloat_data\x18\x64 \x03(\x02\x12\x14\n\x0cinteger_data\x18\x65 \x03(\x05\x12\x13\n\x0bstring_data\x18\x66 \x03(\t\".\n\x08\x44\x61taType\x12\t\n\x05\x46LOAT\x10\x00\x12\x0b\n\x07INTEGER\x10\x01\x12\n\n\x06STRING\x10\x02\"\xff\x01\n\x0cRecordColumn\x12\x0b\n\x03key\x18\x01 \x01(\t\x12/\n\nvalue_type\x18\x02 \x01(\x0e\x32\x1b.lmao.RecordColumn.DataType\x12*\n\x04rows\x18\x03 \x03(\x0b\x32\x1c.lmao.RecordColumn.RecordRow\x1aU\n\tRecordRow\x12\t\n\x01x\x18\x01 \x01(\x03\x12\x12\n\nfloat_data\x18\x64 \x01(\x02\x12\x14\n\x0cinteger_data\x18\x65 \x01(\x05\x12\x13\n\x0bstring_data\x18\x66 \x01(\t\".\n\x08\x44\x61taType\x12\t\n\x05\x46LOAT\x10\x00\x12\x0b\n\x07INTEGER\x10\x01\x12\n\n\x06STRING\x10\x02\"\xca\x01\n\x06RunLog\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x1a\n\x04\x64\x61ta\x18\x02 \x03(\x0b\x32\x0c.lmao.Record\x12\'\n\x0b\x63olumn_data\x18\x03 \x03(\x0b\x32\x12.lmao.RecordColumn\x12&\n\x08log_type\x18\x04 \x01(\x0e\x32\x14.lmao.RunLog.LogType\x12\x12\n\nproject_id\x18\x05 \x01(\t\"(\n\x07LogType\x12\n\n\x06SYSTEM\x10\x00\x12\x07\n\x03NBX\x10\x01\x12\x08\n\x04USER\x10\x02\"\xd7\x01\n\x0c\x41gentDetails\x12$\n\x04type\x18\x06 \x01(\x0e\x32\x16.lmao.AgentDetails.NBX\x12\x14\n\nnbx_job_id\x18\x01 \x01(\tH\x00\x12\x18\n\x0enbx_serving_id\x18\x03 \x01(\tH\x00\x12\x14\n\nnbx_run_id\x18\x02 \x01(\tH\x01\x12\x16\n\x0cnbx_model_id\x18\x04 \x01(\tH\x01\x12\x14\n\x0cworkspace_id\x18\x05 \x01(\t\"\x1b\n\x03NBX\x12\x07\n\x03JOB\x10\x00\x12\x0b\n\x07SERVING\x10\x01\x42\x04\n\x02idB\n\n\x08instance\"\x89\x01\n\x0eInitRunRequest\x12)\n\ragent_details\x18\x01 \x01(\x0b\x32\x12.lmao.AgentDetails\x12\x12\n\ncreated_at\x18\x03 \x01(\x03\x12\x0e\n\x06\x63onfig\x18\x04 \x01(\t\x12\x12\n\nproject_id\x18\x02 \x01(\t\x12\x14\n\x0cproject_name\x18\x05 \x01(\t\"M\n\x04\x46ile\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x12\n\ncreated_at\x18\x02 \x01(\x03\x12\x10\n\x08is_input\x18\x03 \x01(\x08\x12\x11\n\trelic_key\x18\x05 \x01(\t\"<\n\x08\x46ileList\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x19\n\x05\x66iles\x18\x02 \x03(\x0b\x32\n.lmao.File\"\xdb\x02\n\x03Run\x12!\n\x05\x61gent\x18\x01 \x01(\x0b\x32\x12.lmao.AgentDetails\x12\x15\n\rexperiment_id\x18\x02 \x01(\t\x12\x12\n\ncreated_at\x18\x03 \x01(\x03\x12\x10\n\x08\x65nded_at\x18\x04 \x01(\x03\x12\x15\n\rsave_location\x18\x06 \x01(\t\x12!\n\tfile_list\x18\x07 \x01(\x0b\x32\x0e.lmao.FileList\x12\x0e\n\x06\x63onfig\x18\x08 \x01(\t\x12\x12\n\nproject_id\x18\x0b \x01(\t\x12\x14\n\x0cproject_name\x18\x0c \x01(\t\x12 \n\x06status\x18\t \x01(\x0e\x32\x10.lmao.Run.Status\x12\x12\n\nupdated_at\x18\n \x01(\x03\"J\n\x06Status\x12\x0b\n\x07NOT_SET\x10\x00\x12\x0b\n\x07RUNNING\x10\x01\x12\r\n\tCOMPLETED\x10\x02\x12\n\n\x06\x46\x41ILED\x10\x03\x12\x0b\n\x07\x44\x45LETED\x10\x04\"X\n\x13ListProjectsRequest\x12\x14\n\x0cworkspace_id\x18\x01 \x01(\t\x12\x1a\n\x12project_id_or_name\x18\x02 \x01(\t\x12\x0f\n\x07page_no\x18\x03 \x01(\x05\"\xb1\x01\n\x14ListProjectsResponse\x12\x34\n\x08projects\x18\x01 \x03(\x0b\x32\".lmao.ListProjectsResponse.Project\x12\x13\n\x0btotal_pages\x18\x02 \x01(\x05\x1aN\n\x07Project\x12\x12\n\nproject_id\x18\x01 \x01(\t\x12\x14\n\x0cproject_name\x18\x02 \x01(\t\x12\x19\n\x11total_experiments\x18\x03 \x01(\x05\"q\n\x0fListRunsRequest\x12\x14\n\x0cworkspace_id\x18\x01 \x01(\t\x12\x12\n\nproject_id\x18\x02 \x01(\t\x12\x15\n\rexperiment_id\x18\x03 \x01(\t\x12\x0f\n\x07page_no\x18\x04 \x01(\x05\x12\x0c\n\x04\x64\x65sc\x18\x05 \x01(\x08\"a\n\x10ListRunsResponse\x12\x16\n\x0e\x65xperiment_ids\x18\x01 \x03(\t\x12\x0c\n\x04keys\x18\x02 \x03(\t\x12\x12\n\ncreated_at\x18\x03 \x03(\x03\x12\x13\n\x0btotal_pages\x18\x04 \x01(\x05\"y\n\rRunLogRequest\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12\x0b\n\x03key\x18\x02 \x01(\t\x12\x0e\n\x06sample\x18\x03 \x01(\x05\x12\x10\n\x08start_at\x18\x04 \x01(\x05\x12\x0e\n\x06\x65nd_at\x18\x05 \x01(\x05\x12\x12\n\nproject_id\x18\x06 \x01(\t\"/\n\x0b\x41\x63knowledge\x12\x0f\n\x07success\x18\x01 \x01(\x08\x12\x0f\n\x07message\x18\x02 \x01(\t\"V\n\x19GetExperimentTableRequest\x12\x14\n\x0cworkspace_id\x18\x01 \x01(\t\x12\x12\n\nproject_id\x18\x02 \x01(\t\x12\x0f\n\x07page_no\x18\x03 \x01(\x05\"\x8f\x01\n\x0f\x45xperimentTable\x12\x13\n\x0btotal_pages\x18\x01 \x01(\x05\x12\'\n\x04rows\x18\x02 \x03(\x0b\x32\x19.lmao.ExperimentTable.Row\x1a>\n\x03Row\x12\x15\n\rexperiment_id\x18\x01 \x01(\t\x12 \n\x04\x64\x61ta\x18\x02 \x03(\x0b\x32\x12.lmao.RecordColumn2\xbe\x08\n\x04LMAO\x12-\n\x08init_run\x12\x14.lmao.InitRunRequest\x1a\t.lmao.Run\"\x00\x12\x33\n\x11update_run_status\x12\t.lmao.Run\x1a\x11.lmao.Acknowledge\"\x00\x12+\n\x06on_log\x12\x0c.lmao.RunLog\x1a\x11.lmao.Acknowledge\"\x00\x12.\n\x07on_save\x12\x0e.lmao.FileList\x1a\x11.lmao.Acknowledge\"\x00\x12.\n\x0con_train_end\x12\t.lmao.Run\x1a\x11.lmao.Acknowledge\"\x00\x12H\n\rlist_projects\x12\x19.lmao.ListProjectsRequest\x1a\x1a.lmao.ListProjectsResponse\"\x00\x12<\n\tlist_runs\x12\x15.lmao.ListRunsRequest\x1a\x16.lmao.ListRunsResponse\"\x00\x12P\n\x14get_experiment_table\x12\x1f.lmao.GetExperimentTableRequest\x1a\x15.lmao.ExperimentTable\"\x00\x12)\n\x0fget_run_details\x12\t.lmao.Run\x1a\t.lmao.Run\"\x00\x12\x32\n\x0bget_run_log\x12\x13.lmao.RunLogRequest\x1a\x0c.lmao.RunLog\"\x00\x12)\n\nlist_files\x12\t.lmao.Run\x1a\x0e.lmao.FileList\"\x00\x12\x33\n\x11\x64\x65lete_experiment\x12\t.lmao.Run\x1a\x11.lmao.Acknowledge\"\x00\x12\x35\n\x0cinit_serving\x12\x14.lmao.InitRunRequest\x1a\r.lmao.Serving\"\x00\x12\x36\n\x0eon_serving_log\x12\x0f.lmao.LogBuffer\x1a\x11.lmao.Acknowledge\"\x00\x12\x34\n\x0eon_serving_end\x12\r.lmao.Serving\x1a\x11.lmao.Acknowledge\"\x00\x12Q\n\x10list_deployments\x12\x1c.lmao.ListDeploymentsRequest\x1a\x1d.lmao.ListDeploymentsResponse\"\x00\x12H\n\rlist_servings\x12\x19.lmao.ListServingsRequest\x1a\x1a.lmao.ListServingsResponse\"\x00\x12\x35\n\x13get_serving_details\x12\r.lmao.Serving\x1a\r.lmao.Serving\"\x00\x12\x33\n\x0fget_serving_log\x12\r.lmao.Serving\x1a\x0f.lmao.LogBuffer\"\x00\x62\x06proto3')



_LOGBUFFER = DESCRIPTOR.message_types_by_name['LogBuffer']
_SERVINGHTTPLOG = DESCRIPTOR.message_types_by_name['ServingHTTPLog']
_LISTDEPLOYMENTSREQUEST = DESCRIPTOR.message_types_by_name['ListDeploymentsRequest']
_LISTDEPLOYMENTSRESPONSE = DESCRIPTOR.message_types_by_name['ListDeploymentsResponse']
_LISTDEPLOYMENTSRESPONSE_SERVING = _LISTDEPLOYMENTSRESPONSE.nested_types_by_name['Serving']
_LISTSERVINGSREQUEST = DESCRIPTOR.message_types_by_name['ListServingsRequest']
_LISTSERVINGSRESPONSE = DESCRIPTOR.message_types_by_name['ListServingsResponse']
_SERVING = DESCRIPTOR.message_types_by_name['Serving']
_RECORD = DESCRIPTOR.message_types_by_name['Record']
_RECORDCOLUMN = DESCRIPTOR.message_types_by_name['RecordColumn']
_RECORDCOLUMN_RECORDROW = _RECORDCOLUMN.nested_types_by_name['RecordRow']
_RUNLOG = DESCRIPTOR.message_types_by_name['RunLog']
_AGENTDETAILS = DESCRIPTOR.message_types_by_name['AgentDetails']
_INITRUNREQUEST = DESCRIPTOR.message_types_by_name['InitRunRequest']
_FILE = DESCRIPTOR.message_types_by_name['File']
_FILELIST = DESCRIPTOR.message_types_by_name['FileList']
_RUN = DESCRIPTOR.message_types_by_name['Run']
_LISTPROJECTSREQUEST = DESCRIPTOR.message_types_by_name['ListProjectsRequest']
_LISTPROJECTSRESPONSE = DESCRIPTOR.message_types_by_name['ListProjectsResponse']
_LISTPROJECTSRESPONSE_PROJECT = _LISTPROJECTSRESPONSE.nested_types_by_name['Project']
_LISTRUNSREQUEST = DESCRIPTOR.message_types_by_name['ListRunsRequest']
_LISTRUNSRESPONSE = DESCRIPTOR.message_types_by_name['ListRunsResponse']
_RUNLOGREQUEST = DESCRIPTOR.message_types_by_name['RunLogRequest']
_ACKNOWLEDGE = DESCRIPTOR.message_types_by_name['Acknowledge']
_GETEXPERIMENTTABLEREQUEST = DESCRIPTOR.message_types_by_name['GetExperimentTableRequest']
_EXPERIMENTTABLE = DESCRIPTOR.message_types_by_name['ExperimentTable']
_EXPERIMENTTABLE_ROW = _EXPERIMENTTABLE.nested_types_by_name['Row']
_SERVING_STATUS = _SERVING.enum_types_by_name['Status']
_RECORD_DATATYPE = _RECORD.enum_types_by_name['DataType']
_RECORDCOLUMN_DATATYPE = _RECORDCOLUMN.enum_types_by_name['DataType']
_RUNLOG_LOGTYPE = _RUNLOG.enum_types_by_name['LogType']
_AGENTDETAILS_NBX = _AGENTDETAILS.enum_types_by_name['NBX']
_RUN_STATUS = _RUN.enum_types_by_name['Status']
LogBuffer = _reflection.GeneratedProtocolMessageType('LogBuffer', (_message.Message,), {
  'DESCRIPTOR' : _LOGBUFFER,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.LogBuffer)
  })
_sym_db.RegisterMessage(LogBuffer)

ServingHTTPLog = _reflection.GeneratedProtocolMessageType('ServingHTTPLog', (_message.Message,), {
  'DESCRIPTOR' : _SERVINGHTTPLOG,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.ServingHTTPLog)
  })
_sym_db.RegisterMessage(ServingHTTPLog)

ListDeploymentsRequest = _reflection.GeneratedProtocolMessageType('ListDeploymentsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTDEPLOYMENTSREQUEST,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.ListDeploymentsRequest)
  })
_sym_db.RegisterMessage(ListDeploymentsRequest)

ListDeploymentsResponse = _reflection.GeneratedProtocolMessageType('ListDeploymentsResponse', (_message.Message,), {

  'Serving' : _reflection.GeneratedProtocolMessageType('Serving', (_message.Message,), {
    'DESCRIPTOR' : _LISTDEPLOYMENTSRESPONSE_SERVING,
    '__module__' : 'proto.lmao_pb2'
    # @@protoc_insertion_point(class_scope:lmao.ListDeploymentsResponse.Serving)
    })
  ,
  'DESCRIPTOR' : _LISTDEPLOYMENTSRESPONSE,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.ListDeploymentsResponse)
  })
_sym_db.RegisterMessage(ListDeploymentsResponse)
_sym_db.RegisterMessage(ListDeploymentsResponse.Serving)

ListServingsRequest = _reflection.GeneratedProtocolMessageType('ListServingsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTSERVINGSREQUEST,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.ListServingsRequest)
  })
_sym_db.RegisterMessage(ListServingsRequest)

ListServingsResponse = _reflection.GeneratedProtocolMessageType('ListServingsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTSERVINGSRESPONSE,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.ListServingsResponse)
  })
_sym_db.RegisterMessage(ListServingsResponse)

Serving = _reflection.GeneratedProtocolMessageType('Serving', (_message.Message,), {
  'DESCRIPTOR' : _SERVING,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.Serving)
  })
_sym_db.RegisterMessage(Serving)

Record = _reflection.GeneratedProtocolMessageType('Record', (_message.Message,), {
  'DESCRIPTOR' : _RECORD,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.Record)
  })
_sym_db.RegisterMessage(Record)

RecordColumn = _reflection.GeneratedProtocolMessageType('RecordColumn', (_message.Message,), {

  'RecordRow' : _reflection.GeneratedProtocolMessageType('RecordRow', (_message.Message,), {
    'DESCRIPTOR' : _RECORDCOLUMN_RECORDROW,
    '__module__' : 'proto.lmao_pb2'
    # @@protoc_insertion_point(class_scope:lmao.RecordColumn.RecordRow)
    })
  ,
  'DESCRIPTOR' : _RECORDCOLUMN,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.RecordColumn)
  })
_sym_db.RegisterMessage(RecordColumn)
_sym_db.RegisterMessage(RecordColumn.RecordRow)

RunLog = _reflection.GeneratedProtocolMessageType('RunLog', (_message.Message,), {
  'DESCRIPTOR' : _RUNLOG,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.RunLog)
  })
_sym_db.RegisterMessage(RunLog)

AgentDetails = _reflection.GeneratedProtocolMessageType('AgentDetails', (_message.Message,), {
  'DESCRIPTOR' : _AGENTDETAILS,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.AgentDetails)
  })
_sym_db.RegisterMessage(AgentDetails)

InitRunRequest = _reflection.GeneratedProtocolMessageType('InitRunRequest', (_message.Message,), {
  'DESCRIPTOR' : _INITRUNREQUEST,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.InitRunRequest)
  })
_sym_db.RegisterMessage(InitRunRequest)

File = _reflection.GeneratedProtocolMessageType('File', (_message.Message,), {
  'DESCRIPTOR' : _FILE,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.File)
  })
_sym_db.RegisterMessage(File)

FileList = _reflection.GeneratedProtocolMessageType('FileList', (_message.Message,), {
  'DESCRIPTOR' : _FILELIST,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.FileList)
  })
_sym_db.RegisterMessage(FileList)

Run = _reflection.GeneratedProtocolMessageType('Run', (_message.Message,), {
  'DESCRIPTOR' : _RUN,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.Run)
  })
_sym_db.RegisterMessage(Run)

ListProjectsRequest = _reflection.GeneratedProtocolMessageType('ListProjectsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTPROJECTSREQUEST,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.ListProjectsRequest)
  })
_sym_db.RegisterMessage(ListProjectsRequest)

ListProjectsResponse = _reflection.GeneratedProtocolMessageType('ListProjectsResponse', (_message.Message,), {

  'Project' : _reflection.GeneratedProtocolMessageType('Project', (_message.Message,), {
    'DESCRIPTOR' : _LISTPROJECTSRESPONSE_PROJECT,
    '__module__' : 'proto.lmao_pb2'
    # @@protoc_insertion_point(class_scope:lmao.ListProjectsResponse.Project)
    })
  ,
  'DESCRIPTOR' : _LISTPROJECTSRESPONSE,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.ListProjectsResponse)
  })
_sym_db.RegisterMessage(ListProjectsResponse)
_sym_db.RegisterMessage(ListProjectsResponse.Project)

ListRunsRequest = _reflection.GeneratedProtocolMessageType('ListRunsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTRUNSREQUEST,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.ListRunsRequest)
  })
_sym_db.RegisterMessage(ListRunsRequest)

ListRunsResponse = _reflection.GeneratedProtocolMessageType('ListRunsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTRUNSRESPONSE,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.ListRunsResponse)
  })
_sym_db.RegisterMessage(ListRunsResponse)

RunLogRequest = _reflection.GeneratedProtocolMessageType('RunLogRequest', (_message.Message,), {
  'DESCRIPTOR' : _RUNLOGREQUEST,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.RunLogRequest)
  })
_sym_db.RegisterMessage(RunLogRequest)

Acknowledge = _reflection.GeneratedProtocolMessageType('Acknowledge', (_message.Message,), {
  'DESCRIPTOR' : _ACKNOWLEDGE,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.Acknowledge)
  })
_sym_db.RegisterMessage(Acknowledge)

GetExperimentTableRequest = _reflection.GeneratedProtocolMessageType('GetExperimentTableRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETEXPERIMENTTABLEREQUEST,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.GetExperimentTableRequest)
  })
_sym_db.RegisterMessage(GetExperimentTableRequest)

ExperimentTable = _reflection.GeneratedProtocolMessageType('ExperimentTable', (_message.Message,), {

  'Row' : _reflection.GeneratedProtocolMessageType('Row', (_message.Message,), {
    'DESCRIPTOR' : _EXPERIMENTTABLE_ROW,
    '__module__' : 'proto.lmao_pb2'
    # @@protoc_insertion_point(class_scope:lmao.ExperimentTable.Row)
    })
  ,
  'DESCRIPTOR' : _EXPERIMENTTABLE,
  '__module__' : 'proto.lmao_pb2'
  # @@protoc_insertion_point(class_scope:lmao.ExperimentTable)
  })
_sym_db.RegisterMessage(ExperimentTable)
_sym_db.RegisterMessage(ExperimentTable.Row)

_LMAO = DESCRIPTOR.services_by_name['LMAO']
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _LOGBUFFER._serialized_start=59
  _LOGBUFFER._serialized_end=164
  _SERVINGHTTPLOG._serialized_start=167
  _SERVINGHTTPLOG._serialized_end=301
  _LISTDEPLOYMENTSREQUEST._serialized_start=303
  _LISTDEPLOYMENTSREQUEST._serialized_end=394
  _LISTDEPLOYMENTSRESPONSE._serialized_start=397
  _LISTDEPLOYMENTSRESPONSE._serialized_end=574
  _LISTDEPLOYMENTSRESPONSE_SERVING._serialized_start=501
  _LISTDEPLOYMENTSRESPONSE_SERVING._serialized_end=574
  _LISTSERVINGSREQUEST._serialized_start=576
  _LISTSERVINGSREQUEST._serialized_end=689
  _LISTSERVINGSRESPONSE._serialized_start=691
  _LISTSERVINGSRESPONSE._serialized_end=787
  _SERVING._serialized_start=790
  _SERVING._serialized_end=1000
  _SERVING_STATUS._serialized_start=951
  _SERVING_STATUS._serialized_end=1000
  _RECORD._serialized_start=1003
  _RECORD._serialized_end=1192
  _RECORD_DATATYPE._serialized_start=1146
  _RECORD_DATATYPE._serialized_end=1192
  _RECORDCOLUMN._serialized_start=1195
  _RECORDCOLUMN._serialized_end=1450
  _RECORDCOLUMN_RECORDROW._serialized_start=1317
  _RECORDCOLUMN_RECORDROW._serialized_end=1402
  _RECORDCOLUMN_DATATYPE._serialized_start=1146
  _RECORDCOLUMN_DATATYPE._serialized_end=1192
  _RUNLOG._serialized_start=1453
  _RUNLOG._serialized_end=1655
  _RUNLOG_LOGTYPE._serialized_start=1615
  _RUNLOG_LOGTYPE._serialized_end=1655
  _AGENTDETAILS._serialized_start=1658
  _AGENTDETAILS._serialized_end=1873
  _AGENTDETAILS_NBX._serialized_start=1828
  _AGENTDETAILS_NBX._serialized_end=1855
  _INITRUNREQUEST._serialized_start=1876
  _INITRUNREQUEST._serialized_end=2013
  _FILE._serialized_start=2015
  _FILE._serialized_end=2092
  _FILELIST._serialized_start=2094
  _FILELIST._serialized_end=2154
  _RUN._serialized_start=2157
  _RUN._serialized_end=2504
  _RUN_STATUS._serialized_start=2430
  _RUN_STATUS._serialized_end=2504
  _LISTPROJECTSREQUEST._serialized_start=2506
  _LISTPROJECTSREQUEST._serialized_end=2594
  _LISTPROJECTSRESPONSE._serialized_start=2597
  _LISTPROJECTSRESPONSE._serialized_end=2774
  _LISTPROJECTSRESPONSE_PROJECT._serialized_start=2696
  _LISTPROJECTSRESPONSE_PROJECT._serialized_end=2774
  _LISTRUNSREQUEST._serialized_start=2776
  _LISTRUNSREQUEST._serialized_end=2889
  _LISTRUNSRESPONSE._serialized_start=2891
  _LISTRUNSRESPONSE._serialized_end=2988
  _RUNLOGREQUEST._serialized_start=2990
  _RUNLOGREQUEST._serialized_end=3111
  _ACKNOWLEDGE._serialized_start=3113
  _ACKNOWLEDGE._serialized_end=3160
  _GETEXPERIMENTTABLEREQUEST._serialized_start=3162
  _GETEXPERIMENTTABLEREQUEST._serialized_end=3248
  _EXPERIMENTTABLE._serialized_start=3251
  _EXPERIMENTTABLE._serialized_end=3394
  _EXPERIMENTTABLE_ROW._serialized_start=3332
  _EXPERIMENTTABLE_ROW._serialized_end=3394
  _LMAO._serialized_start=3397
  _LMAO._serialized_end=4483
# @@protoc_insertion_point(module_scope)
